/*
Licensed to the Apache Software Foundation (ASF) under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The ASF licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.
*/

#include <stdlib.h>
#include <nuon/bios.h>
#include <nuon/dma.h>
#include <nuon/sprite2.h>
#include <nuon/termemu.h>
#include <nuon/video.h>

#include "Defines.h"
#include "Display.h"

VidDisplay		Display;
VidChannel		MainChannel;
void			*FrameBuffer[NUM_BUFFERS];
SPR_IMAGE_INFO	BackgroundSprite;
SPR_IMAGE_INFO	titleSprite;
int				DrawBuffer	= 0;

extern char BackgroundImage[];
extern char titleImage[];

void DMAClear(char *screen, int dmaflags, int initx, int inity, int wide, int high, int color)
{
	int	x;
	int	y;
	int	lastx;
	int	lasty;

	lastx	= initx + wide;
	lasty	= inity + high;

	for (x = initx; x < lastx; x += 8)
	{
		for (y = inity; y < lasty; y += 8)
		{
			_raw_plotpixel(dmaflags, (void *)screen, (8 << 16)| x, (8 << 16)| y, color);
		}
	}
}

void WaitForVBlank()
{
	static int lastfield;
	int	ret;
	int curfield;

	_VidChangeBase(VID_CHANNEL_MAIN, DMAFLAGS, FrameBuffer[DrawBuffer]);

	// Find out how many fields have been drawn
	curfield = _VidSync(-1);

	// Draw to a new buffer
	DrawBuffer++;
	
	if (DrawBuffer >= NUM_BUFFERS)
	{
		DrawBuffer = 0;
	}

#if NUM_BUFFERS == 2
	// Wait until a new field starts (so it's no longer showing the field we're going to draw on)
	curfield = _VidSync(0);
#endif

	ret	= curfield - lastfield;

	while (0 == ret)
	{
		curfield	= _VidSync(-1);
		
		ret	= curfield - lastfield;
	}

	lastfield	= curfield;
}

bool InitializeDisplay()
{
	int	Loop;

	for (Loop = 0; Loop < NUM_BUFFERS; Loop++)
	{
		FrameBuffer[Loop]	= _MemAlloc(SCREEN_WIDTH * SCREEN_HEIGHT * 4, 512, kMemSDRAM);
	}
	
#if DEBUG
    InitTerminalX(0, (int)FrameBuffer[0], SCREEN_WIDTH, SCREEN_HEIGHT, DMAFLAGS, kBlack);                                                              
#endif
    
	memset(&Display, 0, sizeof(VidDisplay));
	memset(&MainChannel, 0, sizeof(VidChannel));
	
 	Display.dispwidth	= -1;
	Display.dispheight	= -1;
	Display.bordcolor	= 0x20100000;
	Display.progressive	= 0;

	for (Loop = 0; Loop < 6; Loop++)
	{
		Display.reserved[Loop]	= 0;
	}
	
	MainChannel.dmaflags	= DMAFLAGS;
	MainChannel.base		= FrameBuffer[0];
	MainChannel.dest_xoff	= -1;
	MainChannel.dest_yoff	= -1;
	MainChannel.dest_width	= 720;
	MainChannel.dest_height	= 480;
	MainChannel.src_xoff	= 0;
	MainChannel.src_yoff	= 0;
	MainChannel.src_width	= SCREEN_WIDTH;
	MainChannel.src_height	= SCREEN_HEIGHT;
	MainChannel.clut_select	= 0;
	MainChannel.alpha		= 0;
	MainChannel.vfilter		= VID_VFILTER_2TAP;
	MainChannel.hfilter		= VID_HFILTER_4TAP;
	
	for (Loop = 0; Loop < 5; Loop++)
	{
		MainChannel.reserved[Loop]	= 0;
	}

    _VidConfig(&Display, &MainChannel, 0, 0L); 

	DMAClear(FrameBuffer[0], DMAFLAGS, 0, 0, 720, 480, kBlack);
    _VidChangeBase(VID_CHANNEL_MAIN, DMAFLAGS, FrameBuffer[0]);

    SPRInstallTGAImage(BackgroundImage, 1, &BackgroundSprite, 0x00);
    SPRInstallTGAImage(titleImage, 1, &titleSprite, 0x00);
	
	return	true;
}

void DestroyDisplay()
{
}

void UpdateDisplay(int back)
{
	if (back == 1){
	SPRBlitter(BackgroundSprite.img, BackgroundSprite.dmaflags, 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT, FrameBuffer[DrawBuffer], DMAFLAGS, 
		SCREEN_WIDTH / 2, SCREEN_HEIGHT / 2, 0x20000 * SCREEN_WIDTH / 720, 0x20000 * SCREEN_HEIGHT / 480, 0, 0, 0, SCREEN_WIDTH - 1, 
		SCREEN_HEIGHT - 1, kSpriteSimple, 0);
	}
	else if (back == 0){
	SPRBlitter(titleSprite.img, titleSprite.dmaflags, 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT, FrameBuffer[DrawBuffer], DMAFLAGS, 
		SCREEN_WIDTH / 2, SCREEN_HEIGHT / 2, 0x20000 * SCREEN_WIDTH / 720, 0x20000 * SCREEN_HEIGHT / 480, 0, 0, 0, SCREEN_WIDTH - 1, 
		SCREEN_HEIGHT - 1, kSpriteSimple, 0);
	}
	
    SPRSetDestScreen(FrameBuffer[DrawBuffer], DMAFLAGS, 0, 0, SCREEN_WIDTH - 1, SCREEN_HEIGHT - 1, kBlack);

	UpdateSprites();

	WaitForVBlank();
}
