/*
Licensed to the Apache Software Foundation (ASF) under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The ASF licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.
*/

#include <stdio.h>
#include <nuon/dma.h>
#include <nuon/mml2d.h>

#include "Defines.h"
#include "Sprite.h"

// Sprite list
sSprite	SpriteList[MAX_SPRITES];

SPR_IMAGE_INFO	carreInfo;
SPR_IMAGE_INFO	scoreInfo;
SPR_IMAGE_INFO	readyInfo;
SPR_IMAGE_INFO	handInfo;
SPR_IMAGE_INFO	carreInfo2;

extern char carreImage[];
extern char scoreImage[];
extern char readyImage[];
extern char handImage[];

extern	void	*FrameBuffer[NUM_BUFFERS];

bool InitializeSprites()
{
	int	iLoop;
	
    // Initialize sprite system
    SPRInit(0, 2, 16, 0);

	// Set destination screen
	SPRSetDestScreen(FrameBuffer[1], DMAFLAGS,  0,  0,  SCREEN_WIDTH - 1,  SCREEN_HEIGHT - 1, kBlack);
    
	// Install images
	if (-1 == SPRInstallTGAImage(carreImage, 1, &carreInfo, 0xFF00FF00))
	{
		return	false;
	}

	if (-1 == SPRInstallTGAImage(scoreImage, 1, &scoreInfo, 0xFF00FF00))
	{
		return	false;
	}
	
	if (-1 == SPRInstallTGAImage(readyImage, 1, &readyInfo, 0xFF00FF00))
	{
		return	false;
	}


	if (-1 == SPRInstallTGAImage(handImage, 1, &handInfo, 0xFF00FF00))
	{
		return	false;
	}
	
	// Install images
	if (-1 == SPRInstallTGAImage(carreImage, 1, &carreInfo2, 0xFF00FF00))
	{
		return	false;
	}



	// Set all sprites to unused
	for (iLoop = 0; iLoop < MAX_SPRITES; ++iLoop)
	{
		SpriteList[iLoop].bUsed		= false;
		SpriteList[iLoop].pSprite	= NULL;
	}

	return	true;
}

void UpdateSprites(int iScreen)
{
	// Draw the sprite don't clear the screen, and wait for it to complete.
	SPRDraw(false, true);
}

int	AddSprite(int iImageIndex, int iWidth, int iHeight)
{
	// Find the next available sprite
	int	iSpriteIndex	= FindAvailableSprite();

	if (iSpriteIndex >= 0)
	{
		SPR_IMAGE_INFO	*pImageInfo;
		
		switch (iImageIndex)
		{
			case 0:
				pImageInfo	= &carreInfo;
				
				break;

			case 1:
				pImageInfo	= &scoreInfo;
				
				break;

			case 2:
				pImageInfo	= &readyInfo;
				
				break;

			case 3:
				pImageInfo	= &handInfo;
				
				break;
			case 4:
				pImageInfo	= &carreInfo2;
				
				break;
		}
		
		SpriteList[iSpriteIndex].bUsed		= true;
		SpriteList[iSpriteIndex].iWidth		= iWidth;
		SpriteList[iSpriteIndex].iFrame		= 0;
		SpriteList[iSpriteIndex].iHeight	= iHeight;
		
		// Create sprite
		SpriteList[iSpriteIndex].pSprite	= SPRCreateSprite(pImageInfo, 0, 0, SpriteList[iSpriteIndex].iWidth, SpriteList[iSpriteIndex].iHeight);

		if (NULL == SpriteList[iSpriteIndex].pSprite)
		{
			return	-1;
		}
	}

	return	iSpriteIndex;
}

void RemoveSprite(int iIndex)
{
	// Remove the sprite from the display list
	SPRRemoveSprite(SpriteList[iIndex].pSprite);

	// Delete the sprite
	SPRDeleteSprite(SpriteList[iIndex].pSprite);
	
	// Mark the sprite as unused
	SpriteList[iIndex].bUsed	= false;
}

void SetSpritePosition(int iIndex, int iX, int iY)
{
	SpriteList[iIndex].ptPosition.iX	= iX;
	SpriteList[iIndex].ptPosition.iY	= iY;

	SPRSetSpriteXY(SpriteList[iIndex].pSprite, iX, iY);
}

void SetSpriteVisible(int iIndex, bool bVisible)
{
	if (true == bVisible)
	{
		// Add the sprite to display list
		if (-1 == SPRAddSprite(SpriteList[iIndex].pSprite, SpriteList[iIndex].ptPosition.iX, SpriteList[iIndex].ptPosition.iY, 0, 0x10000,
			0x10000, kSpriteSimpleTrans, kBlack, 1))
		{
			SPRModifySprite(SpriteList[iIndex].pSprite, SpriteList[iIndex].ptPosition.iX, SpriteList[iIndex].ptPosition.iY, 0, 0x10000,
				0x10000, kSpriteSimpleTrans, kBlack, 1);
		}
		
		// Reset the frame
		SPRSetSpriteSource(SpriteList[iIndex].pSprite, SpriteList[iIndex].iWidth * SpriteList[iIndex].iFrame, 0, SpriteList[iIndex].iWidth, 
			SpriteList[iIndex].iHeight);
	}
	
	else
	{
		// Remove the sprite from the display list
		SPRRemoveSprite(SpriteList[iIndex].pSprite);
	}
}

void SetSpriteFrame(int iIndex, int iFrame)
{
	SPRSetSpriteSource(SpriteList[iIndex].pSprite, SpriteList[iIndex].iWidth * iFrame, 0, SpriteList[iIndex].iWidth, SpriteList[iIndex].iHeight);

	SpriteList[iIndex].iFrame	= iFrame;
}

int FindAvailableSprite()
{
	int	iLoop;
	
	for (iLoop = 0; iLoop < MAX_SPRITES; ++iLoop)
	{
		if (false == SpriteList[iLoop].bUsed)
		{
			return	iLoop;
		}
	}

	return	-1;
}

int GetSpriteWidth(int iIndex)
{
	return	SpriteList[iIndex].iWidth;
}

int GetSpriteHeight(int iIndex)
{
	return	SpriteList[iIndex].iHeight;
}
