/*
Licensed to the Apache Software Foundation (ASF) under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The ASF licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.
*/

#ifndef __SPRITE_H_
#define __SPRITE_H_

#include <nuon/sprite2.h>

#include "stdbool.h"

// Maximum number of sprites
#define	MAX_SPRITES	64

typedef struct
{
	// Sprite x and y
	int	iX;
	int	iY;
} Point;

// Sprite element
typedef struct
{
	// Position
	Point	ptPosition;
	
	// Width
	int		iWidth;
	
	// Height
	int		iHeight;
	
	// Frame
	int		iFrame;
	
	// Sprite pointer
	SPRITE	*pSprite;
	
	// Is this sprite used
	bool	bUsed;
} sSprite;

// Initialize sprites
bool InitializeSprites();

// Update sprites
void UpdateSprites();

// Add sprite
int	AddSprite(int iImageIndex, int iWidth, int iHeight);

// Remove sprite
void RemoveSprite(int iIndex);

// Set sprite position
void SetSpritePosition(int iIndex, int iX, int iY);

// Set sprite frame
void SetSpriteFrame(int iIndex, int iFrame);

// Set sprite visibility
void SetSpriteVisible(int iIndex, bool bVisible);

// Find available sprite
int FindAvailableSprite();

// Get sprite width
int GetSpriteWidth(int iIndex);

// Get sprite height
int GetSpriteHeight(int iIndex);

#endif
